datablock StaticShapeData(WaypointShape)
{
	category = "Shapes";
	shapeFile = "./waypoint.dts";
};

datablock AudioProfile(makeWaypointSound)
{
   filename    = "./makeWaypoint.wav";
   description = AudioClose3d;
   preload = true;
};

datablock AudioProfile(teleportWaypointSound)
{
   filename    = "./teleport.wav";
   description = AudioClose3d;
   preload = true;
};

datablock AudioProfile(deleteWaypointSound)
{
   filename    = "./deleteWaypoint.wav";
   description = AudioClose3d;
   preload = true;
};

datablock ParticleData(waypointExplosionParticle)
{
	dragCoefficient		= 10;
	gravityCoefficient	= 0.0;
	inheritedVelFactor	= 0.2;
	constantAcceleration = 0.0;
	lifetimeMS			  = 300;
	lifetimeVarianceMS	= 290;
	spinSpeed		= 0.0;
	spinRandomMin		= -150.0;
	spinRandomMax		= 150.0;
	textureName			 = "base/data/particles/thinRing";
	colors[0]	  = "0.4 0.9 0.4 0.9";
	colors[1]	  = "0.0 1.0 0.0 0.9";
	colors[2]	  = "0.0 1.0 0.6 1.0";

	sizes[0]		= 0.0;
	sizes[1]		= 0.7;
	sizes[2]		= 2.0;

	times[0] = 0.0;
	times[1] = 0.9;
	times[2] = 1.0;
};

datablock ParticleEmitterData(waypointExplosionEmitter)
{
	lifeTimeMS = 50;

	ejectionPeriodMS = 1;
	periodVarianceMS = 0;
	ejectionVelocity = 15;
	velocityVariance = 4.0;
	ejectionOffset	= 1.25;
	thetaMin			= 0;
	thetaMax			= 180;
	phiReferenceVel  = 0;
	phiVariance		= 360;
	overrideAdvance = false;
	particles = "waypointExplosionParticle";

	uiName = "Horse Ray Explosion";
};

datablock ExplosionData(waypointExplosion)
{
	explosionShape = "Add-Ons/Weapon_Rocket_Launcher/explosionsphere1.dts";
	
	emitter[0] = horseRayExplosionEmitter;
	soundProfile = makeWaypointSound;
	
	lifeTimeMS = 150;
	faceViewer	  = true;
	explosionScale = "1 1 1";
	shakeCamera = false;
};

datablock ProjectileData(waypointExplosionProjectile)
{
	lifetime						= 10;
	fadeDelay						= 10;
	explodeondeath						= true;
	explosion						= waypointExplosion;
};

// rtb stuff (code from hackplant (also made by me))
if(isFile("Add-Ons/System_ReturnToBlockland/server.cs"))
{
	if(!$Waypoints::PrefsSet)
	{
		if(!$RTB::RTBR_ServerControl_Hook)
		{
			exec("Add-Ons/System_ReturnToBlockland/RTBR_ServerControl_Hook.cs");
		}
		RTB_registerPref("Enable Waypoints?", "Waypoints", "$Waypoints::pref::Enabled", "bool", "Script_Waypoints", 0, 0, 0);
		RTB_registerPref("Admin Only?", "Waypoints", "$Waypoints::pref::AdminOnly", "bool", "Script_Waypoints", 0, 0, 0);
		RTB_registerPref("Waypoints Timeout", "Waypoints", "Waypoints::pref::Timeout", "int 0 60", "Script_Waypoints", 1, 0, 0);
		$Waypoints::PrefsSet = 1;
	}
}

// functions
function serverCmdWaypoint(%client, %wayID)
{
	if(%wayID $= "")
		return messageClient(%client, '', "\c6You did not input the \c4wayID \c6feild!");
		
	if(!$Waypoints::pref::Enabled)
		return messageClient(%client, '', "\c6Waypoints are not currently enabled.");
		
	if($HackPlant::pref::AdminOnly && !%client.isAdmin)
		return messageClient(%client, '', "\c6Waypoints are currently admin-only.");
		
	if($Waypoints::pref::Timeout > 0)
	{
		%len = $Waypoints::pref::Timeout * 1000;
			
		if(!%client.isAdmin && getSimTime() - %client.player.lastWaypointDeployTime < %len)
		{
			parent::serverCmdPlantBrick(%client);
				
			// used the code from hackplant
			%timeout = getSimTime() - %client.player.lastWaypointDeployTime;
			%timeout = %len - %timeout;
			%timeout /= 1000;
			%timeout = mCeil(%timeout);
			%timeout = %timeout @ "\c4" @ (%timeout > 1 ? " seconds":" second");			
			%client.player.tempBrick.playSound(hackPlantErrorSound);
			commandtoClient(%client,'CenterPrint',"\c6You must wait" SPC %timeout SPC "\c6before creating another waypoint.",3);
			return;
		}
	}
		
	if(%client.wayPointNameCount $= "")
		%client.wayPointNameCount = 0;
		
	if(%wayID $= %client.wayPointName[%client.wayPointNameCount - 1])
	{
		%client.waypointPos[%wayID] = "";
		
		%client.wayPointName[%client.wayPointNameCount] = "";
		
		%client.wayPointNameCount--;
	}
		
	%player = %client.player;	
	
	%client.waypointPos[%wayID] = %player.getPosition();
	
	%client.wayPointName[%client.wayPointNameCount] = %wayID;
	
	%client.wayPointNameCount++;
	
	%player.spawnWaypoint();
	
	%client.player.lastWaypointDeployTime = getSimTime();
	
	messageClient(%client, '', "\c6Created waypoint at WayID of\c4" SPC %wayID @ "\c6.");
}

function serverCmdgotoWaypoint(%client, %wayID)
{
	if(%wayID $= "")
		return messageClient(%client, '', "\c6You did not input the \c4wayID \c6feild!");
		
	if(%client.waypointPos[%wayID] $= "")
		return messageClient(%client, '', "\c6There is no waypoint at this \c4wayID\c6!");
	
	%player = %client.player;
	
	serverPlay3D(teleportWaypointSound,%player.getPosition());
	
	%player.setTransform(%client.waypointPos[%wayID]);
	
	messageClient(%client, '', "\c6Teleported to waypoint at WayID of\c4" SPC %wayID @ "\c6.");
	
	%player.spawnWaypoint();
}

function serverCmdDeleteWaypoint(%client, %wayID)
{
	if(%wayID $= "")
		return messageClient(%client, '', "\c6You did not input the \c4wayID \c6feild!");
		
	if(%client.waypointPos[%wayID] $= "")
		return messageClient(%client, '', "\c6There is no waypoint at this \c4wayID\c6!");
	
	%client.waypointPos[%wayID] = "";
	
	%client.wayPointName[%client.wayPointNameCount] = "";
	
	%client.wayPointNameCount--;
	
	%player = %client.player;
	
	serverPlay3D(deleteWaypointSound,%player.getPosition());
	
	messageClient(%client, '', "\c6Deleted waypoint at WayID of\c4" SPC %wayID @ "\c6.");
}

function Player::spawnWaypoint(%this)
{
	%waypoint = new StaticShape() 
	{
		position = %this.getHackPosition();
		rotation = getWords(%this.getTransform(), 3, 6);
		datablock = "WaypointShape";
		scale = "1 1 1";
	}; 
	
	%waypoint.playThread(0, "spin");
	%waypoint.schedule(3000,delete);
	
	%p = new Projectile()
	{
		dataBlock = "waypointExplosionProjectile";
		initialPosition = %waypoint.getPosition();
		sourceObject = %waypoint;
	};
	MissionCleanup.add(%p);	
	return %p;
}

function serverCmdListWaypoints(%client)
{
	for (%i = 0; %i < %client.wayPointNameCount; %i++)
	{
		%wayID = %client.wayPointName[%i];
		%pos = %client.wayPointPos[%wayID];	
		messageClient(%client, '', "\c6Waypoint at \c4" SPC %pos SPC "\c6 with WayID of\c4" SPC %wayID @ "\c6.");	
	}
}

function serverCmdDeleteAllWaypoints(%client)
{
	for (%i = 0; %i < %client.wayPointNameCount; %i++)
	{
		%wayID = %client.wayPointName[%i];
		%pos = %client.wayPointPos[%wayID];	
		%client.waypointPos[%wayID] = "";
		%client.wayPointName[%i] = "";
	}
	messageClient(%client, '', "\c6All \c4waypoints \c6deleted.");
	
	%player = %client.player;
	
	serverPlay3D(deleteWaypointSound,%player.getPosition());
	
	%client.wayPointNameCount = 0;	
}

function serverCmdWaypointHelp(%client)
{
	messageClient(%client, '', "\c6/Waypoint [wayID]\c4 - creates a waypoint with the name of [wayID] at your current location.");
	
	messageClient(%client, '', "\c6/DeleteWaypoint [wayID]\c4 - deletes the waypoint with the name of [wayID].");
	
	messageClient(%client, '', "\c6/GotoWaypoint [wayID]\c4 - teleports you to the waypoint with the name of [wayID].");
	
	messageClient(%client, '', "\c6/ListWaypoints\c4 - lists all of your waypoints.");
	
	messageClient(%client, '', "\c6/DeleteAllWaypoints\c4 - deletes all of your waypoints.");
}
